/*
Copyright © 2023 Philipp Wolfer <phw@uploadedlobster.com>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package funkwhale

import (
	"errors"
	"strconv"

	"github.com/go-resty/resty/v2"
	"go.uploadedlobster.com/scotty/internal/version"
	"go.uploadedlobster.com/scotty/pkg/ratelimit"
)

const MaxItemsPerGet = 50

type Client struct {
	HTTPClient *resty.Client
	token      string
}

func NewClient(serverURL string, token string) Client {
	client := resty.New()
	client.SetBaseURL(serverURL)
	client.SetAuthScheme("Bearer")
	client.SetAuthToken(token)
	client.SetHeader("Accept", "application/json")
	client.SetHeader("User-Agent", version.UserAgent())

	// Handle rate limiting (see https://docs.funkwhale.audio/developer/api/rate-limit.html)
	ratelimit.EnableHTTPHeaderRateLimit(client, "Retry-After")

	return Client{
		HTTPClient: client,
		token:      token,
	}
}

func (c Client) GetHistoryListenings(user string, page int, perPage int) (result ListeningsResult, err error) {
	const path = "/api/v1/history/listenings"
	response, err := c.HTTPClient.R().
		SetQueryParams(map[string]string{
			"username":  user,
			"page":      strconv.Itoa(page),
			"page_size": strconv.Itoa(perPage),
			"ordering":  "-creation_date",
		}).
		SetResult(&result).
		Get(path)

	if !response.IsSuccess() {
		err = errors.New(response.String())
		return
	}
	return
}

func (c Client) GetFavoriteTracks(page int, perPage int) (result FavoriteTracksResult, err error) {
	const path = "/api/v1/favorites/tracks"
	response, err := c.HTTPClient.R().
		SetQueryParams(map[string]string{
			"page":      strconv.Itoa(page),
			"page_size": strconv.Itoa(perPage),
			"ordering":  "-creation_date",
		}).
		SetResult(&result).
		Get(path)

	if !response.IsSuccess() {
		err = errors.New(response.String())
		return
	}
	return
}
